﻿using C1.WPF.FlexGrid;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WPF_C1FlexGrid_Sorting
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ObservableCollection<Employee> emp = new ObservableCollection<Employee>();
        public MainWindow()
        {
            InitializeComponent();

            emp.Add(new Employee() { ID = 101, FirstName = "John",LastName="Carter", Department = "Dev Tools", Location = "US" });
            emp.Add(new Employee() { ID = 102, FirstName = "Mike", LastName = "Smith", Department = "Development", Location = "UK" });
            emp.Add(new Employee() { ID = 103, FirstName = "Fuller", LastName = "Williams", Department = "R&D", Location = "US" });
            emp.Add(new Employee() { ID = 104, FirstName = "Lilly", LastName = "Williams", Department = "HR", Location = "Japan" });
            emp.Add(new Employee() { ID = 105, FirstName = "Prabhat", LastName = "Kumar", Department = "Accounts", Location = "India" });
            emp.Add(new Employee() { ID = 106, FirstName = "Richa", LastName = "Jindal", Department = "HR", Location = "India" });
            emp.Add(new Employee() { ID = 107, FirstName = "Miller", LastName = "Allen", Department = "Dev Tools", Location = "US" });
            emp.Add(new Employee() { ID = 108, FirstName = "Sam", LastName = "Paul", Department = "Dev Tools", Location = "UK" });

            this.DataContext = emp;

            c1FlexGrid.Click += c1FlexGrid_Click;
            c1FlexGrid.CellEditEnded += C1FlexGrid_CellEditEnded;
            c1FlexGrid.AllowSorting = false;
           
        }

        private void C1FlexGrid_CellEditEnded(object sender, CellEditEventArgs e)
        {
            IEditableCollectionView EditableCollectionView = c1FlexGrid.CollectionView as IEditableCollectionView;
            if (EditableCollectionView != null && EditableCollectionView.CurrentEditItem != null)
            {
                EditableCollectionView.CommitEdit();
            }
            else
            {
                EditableCollectionView.CommitNew();
            }
        }

        void c1FlexGrid_Click(object sender, MouseButtonEventArgs e)
        {
            var ht = c1FlexGrid.HitTest(e);

            // sort column when user clicks a column header
            if (ht.CellType == CellType.ColumnHeader)
            {
                var col = c1FlexGrid.Columns[ht.Column];
                var sortDirection = (col.Tag is ListSortDirection) && (ListSortDirection)col.Tag ==

                ListSortDirection.Ascending ? ListSortDirection.Descending : ListSortDirection.Ascending;
                col.Tag = sortDirection;
                //implement sort logic on this column
                SortColumn(col, sortDirection);
                foreach (C1.WPF.FlexGrid.Column column in c1FlexGrid.Columns)
                {
                    if ((column.Index != col.Index))
                    {
                        column.Tag = null;
                    }
                }
            }

            // toggle Boolean cells even when the click is outside the box
            if (ht.CellType == CellType.Cell)
            {
                // check that it is a Boolean cell
                if (c1FlexGrid.Columns[ht.Column].DataType == typeof(bool))
                {
                    // check that it is a single cell (not when extending the selection)
                    if (c1FlexGrid.Selection.IsSingleCell)
                    {
                        // update Boolean value
                        var value = c1FlexGrid[ht.Row, ht.Column];
                        c1FlexGrid[ht.Row, ht.Column] = value is bool && Convert.ToBoolean(value) ? false : true;

                        // we're done with this event
                        e.Handled = true;
                    }
                }
            }

           
        }

        // sort a column given a sort direction
        private void SortColumn(Column col, ListSortDirection sortDirection)
        {
            // build row list
            var list = new List<Row>();
            foreach (C1.WPF.FlexGrid.Row row in c1FlexGrid.Rows)
            {
                list.Add(row);
            }

            // sort row list
            list.Sort(new RowComparer(col, sortDirection));

            // re-populate grid with sorted rows
            c1FlexGrid.Rows.Clear();
            foreach (C1.WPF.FlexGrid.Row row in list)
            {
                c1FlexGrid.Rows.Add(row);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            SortColumn(c1FlexGrid.Columns["FirstName"], (ListSortDirection)c1FlexGrid.Columns["FirstName"].Tag);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //design column column header (post-sort)
            c1FlexGrid.CellFactory = new SortCellFactory();
            c1FlexGrid.Columns["FirstName"].Tag = ListSortDirection.Ascending;
            SortColumn(c1FlexGrid.Columns["FirstName"], (ListSortDirection)c1FlexGrid.Columns["FirstName"].Tag);
        }
    }

    public class Employee
    {
        public int ID { get; set; }
        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string Department { get; set; }
        public string Location { get; set; }
    }
}
